@@ -553,29 +553,11 @@ MYSQLND_METHOD(mysqlnd_conn_data, get_scheme)(MYSQLND_CONN_DATA * conn, MYSQLND_
 			port = 3306;
 		}
 
+		/* ipv6 addresses are in the format [address]:port */
 		if (hostname.s[0] != '[' && mysqlnd_fast_is_ipv6_address(hostname.s)) {
-			/* IPv6 without square brackets so without port */
 			transport.l = mnd_sprintf(&transport.s, 0, "tcp://[%s]:%u", hostname.s, port);
 		} else {
-			char *p;
-
-			/* IPv6 addresses are in the format [address]:port */
-			if (hostname.s[0] == '[') { /* IPv6 */
-				p = strchr(hostname.s, ']');
-				if (p && p[1] != ':') {
-					p = NULL;
-				}
-			} else { /* IPv4 or name */
-				p = strchr(hostname.s, ':');
-			}
-			/* Could already contain a port number, in which case we should not add an extra port.
-			 * See GH-8978. In a port doubling scenario, the first port would be used so we do the same to keep BC. */
-			if (p) {
-				/* TODO: Ideally we should be able to get rid of this workaround in the future. */
-				transport.l = mnd_sprintf(&transport.s, 0, "tcp://%s", hostname.s);
-			} else {
-				transport.l = mnd_sprintf(&transport.s, 0, "tcp://%s:%u", hostname.s, port);
-			}
+			transport.l = mnd_sprintf(&transport.s, 0, "tcp://%s:%u", hostname.s, port);
 		}
 	}
 	DBG_INF_FMT("transport=%s", transport.s? transport.s:"OOM");
@@ -611,7 +593,7 @@ MYSQLND_METHOD(mysqlnd_conn_data, connect)(MYSQLND_CONN_DATA * conn,
 
 	DBG_INF_FMT("host=%s user=%s db=%s port=%u flags=%u persistent=%u state=%u",
 				hostname.s?hostname.s:"", username.s?username.s:"", database.s?database.s:"", port, mysql_flags,
-				conn? conn->persistent:0, conn? (int)GET_CONNECTION_STATE(&conn->state):-1);
+				conn->persistent, (int)GET_CONNECTION_STATE(&conn->state));
 
 	if (GET_CONNECTION_STATE(&conn->state) > CONN_ALLOCED) {
 		DBG_INF("Connecting on a connected handle.");
@@ -1478,10 +1460,6 @@ MYSQLND_METHOD(mysqlnd_conn_data, set_client_option)(MYSQLND_CONN_DATA * const c
 		}
 		case MYSQL_READ_DEFAULT_FILE:
 		case MYSQL_READ_DEFAULT_GROUP:
-#ifdef WHEN_SUPPORTED_BY_MYSQLI
-		case MYSQL_SET_CLIENT_IP:
-		case MYSQL_REPORT_DATA_TRUNCATION:
-#endif
 			/* currently not supported. Todo!! */
 			break;
 		case MYSQL_SET_CHARSET_NAME:
@@ -1509,18 +1487,6 @@ MYSQLND_METHOD(mysqlnd_conn_data, set_client_option)(MYSQLND_CONN_DATA * const c
 				conn->options->protocol = *(unsigned int*) value;
 			}
 			break;
-#ifdef WHEN_SUPPORTED_BY_MYSQLI
-		case MYSQL_SET_CHARSET_DIR:
-		case MYSQL_OPT_RECONNECT:
-			/* we don't need external character sets, all character sets are
-			   compiled in. For compatibility we just ignore this setting.
-			   Same for protocol, we don't support old protocol */
-		case MYSQL_OPT_USE_REMOTE_CONNECTION:
-		case MYSQL_OPT_USE_EMBEDDED_CONNECTION:
-		case MYSQL_OPT_GUESS_CONNECTION:
-			/* todo: throw an error, we don't support embedded */
-			break;
-#endif
 		case MYSQLND_OPT_MAX_ALLOWED_PACKET:
 			if (*(unsigned int*) value > (1<<16)) {
 				conn->options->max_allowed_packet = *(unsigned int*) value;
@@ -1556,12 +1522,6 @@ MYSQLND_METHOD(mysqlnd_conn_data, set_client_option)(MYSQLND_CONN_DATA * const c
 				DBG_INF_FMT("%d left", zend_hash_num_elements(conn->options->connect_attr));
 			}
 			break;
-#ifdef WHEN_SUPPORTED_BY_MYSQLI
-		case MYSQL_SHARED_MEMORY_BASE_NAME:
-		case MYSQL_OPT_USE_RESULT:
-		case MYSQL_SECURE_AUTH:
-			/* not sure, todo ? */
-#endif
 		default:
 			ret = FAIL;
 	}
