@@ -1,13 +1,11 @@
 /*
   +----------------------------------------------------------------------+
-  | PHP Version 7                                                        |
-  +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2018 The PHP Group                                |
+  | Copyright (c) The PHP Group                                          |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
   | available through the world-wide-web at the following url:           |
-  | http://www.php.net/license/3_01.txt                                  |
+  | https://www.php.net/license/3_01.txt                                 |
   | If you did not receive a copy of the PHP license and are unable to   |
   | obtain it through the world-wide-web, please send a note to          |
   | license@php.net so we can mail you a copy immediately.               |
@@ -17,6 +15,8 @@
   +----------------------------------------------------------------------+
 */
 
+/* internal header; not supposed to be installed */
+
 #ifndef PHP_PDO_DBLIB_INT_H
 #define PHP_PDO_DBLIB_INT_H
 
@@ -24,7 +24,7 @@
 # define PDO_DBLIB_FLAVOUR "Generic DB-lib"
 #endif
 
-#if PHP_DBLIB_IS_MSSQL
+#ifdef PHP_DBLIB_IS_MSSQL
 # include <sqlfront.h>
 # include <sqldb.h>
 
@@ -100,7 +100,7 @@ int pdo_dblib_error_handler(DBPROCESS *dbproc, int severity, int dberr,
 	int oserr, char *dberrstr, char *oserrstr);
 
 int pdo_dblib_msg_handler(DBPROCESS *dbproc, DBINT msgno, int msgstate,
-	int severity, char *msgtext, char *srvname, char *procname, DBUSMALLINT line);
+	int severity, char *msgtext, char *srvname, char *procname, int line);
 
 extern const pdo_driver_t pdo_dblib_driver;
 extern const struct pdo_stmt_methods dblib_stmt_methods;
@@ -145,12 +145,13 @@ ZEND_BEGIN_MODULE_GLOBALS(dblib)
 	char sqlstate[6];
 ZEND_END_MODULE_GLOBALS(dblib)
 
-#ifdef ZTS
-# define DBLIB_G(v) TSRMG(dblib_globals_id, zend_dblib_globals *, v)
-#else
-# define DBLIB_G(v) (dblib_globals.v)
+#if defined(ZTS) && (defined(COMPILE_DL_PDO_DBLIB) || defined(COMPILE_DL_PDO_MSSQL))
+ZEND_TSRMLS_CACHE_EXTERN()
 #endif
 
+ZEND_EXTERN_MODULE_GLOBALS(dblib)
+#define DBLIB_G(v) ZEND_MODULE_GLOBALS_ACCESSOR(dblib, v)
+
 ZEND_EXTERN_MODULE_GLOBALS(dblib)
 
 enum {
